@@ -5,7 +5,7 @@ use utf8;
 
 package Neo4j::Driver::Transaction;
 # ABSTRACT: Logical container for an atomic unit of work
-$Neo4j::Driver::Transaction::VERSION = '0.35';
+$Neo4j::Driver::Transaction::VERSION = '0.36';
 
 use Carp qw(croak);
 our @CARP_NOT = qw(
@@ -21,15 +21,18 @@ use Neo4j::Driver::Result;
 
 sub new {
 	# uncoverable pod (private method)
-	my ($class, $session) = @_;
+	my ($class, $session, $mode) = @_;
 	
+	my $events = $session->{driver}->{plugins};
 	my $transaction = {
 		cypher_params_v2 => $session->{cypher_params_v2},
 		net => $session->{net},
+		mode => $mode,
 		unused => 1,  # for HTTP only
 		closed => 0,
 		return_graph => 0,
 		return_stats => 1,
+		error_handler => sub { $events->trigger(error => shift) },
 	};
 	
 	return bless $transaction, $class;
@@ -114,7 +117,7 @@ sub _begin {
 	croak "Concurrent transactions are unsupported in Bolt (there is already an open transaction in this session)" if $self->{net}->{active_tx};
 	
 	try {
-		$self->{bolt_txn} = $self->{net}->_new_tx;
+		$self->{bolt_txn} = $self->{net}->_new_tx($self);
 	}
 	catch {
 		die $_ if $_ !~ m/\bprotocol version\b/i;  # Bolt v1/v2
@@ -152,6 +155,7 @@ sub commit {
 	my ($self) = @_;
 	
 	croak 'Transaction already closed' unless $self->is_open;
+	croak 'Use `return` to commit a managed transaction' if $self->{managed};
 	
 	if ($self->{bolt_txn}) {
 		$self->{bolt_txn}->commit;
@@ -168,6 +172,7 @@ sub rollback {
 	my ($self) = @_;
 	
 	croak 'Transaction already closed' unless $self->is_open;
+	croak 'Explicit rollback of a managed transaction' if $self->{managed};
 	
 	if ($self->{bolt_txn}) {
 		$self->{bolt_txn}->rollback;
@@ -251,6 +256,8 @@ sub _run_autocommit {
 sub commit {
 	my ($self) = @_;
 	
+	croak 'Use `return` to commit a managed transaction' if $self->{managed};
+	
 	$self->_run_autocommit;
 }
 
@@ -259,6 +266,7 @@ sub rollback {
 	my ($self) = @_;
 	
 	croak 'Transaction already closed' unless $self->is_open;
+	croak 'Explicit rollback of a managed transaction' if $self->{managed};
 	
 	$self->{net}->_request($self, 'DELETE') if $self->{transaction_endpoint};
 	$self->{closed} = 1;
@@ -288,7 +296,7 @@ Neo4j::Driver::Transaction - Logical container for an atomic unit of work
 
 =head1 VERSION
 
-version 0.35
+version 0.36
 
 =head1 SYNOPSIS
 
@@ -324,8 +332,8 @@ execution of a statement to have completed, you need to use the
 L<Result|Neo4j::Driver::Result>, for example by calling
 one of the methods C<fetch()>, C<list()> or C<summary()>.
 
-To create a new (unmanaged) transaction, call
-L<Neo4j::Driver::Session/"begin_transaction">.
+Neo4j drivers allow the creation of different kinds of transactions.
+See L<Neo4j::Driver::Session> for details.
 
 =head1 METHODS
 
@@ -335,7 +343,7 @@ L<Neo4j::Driver::Transaction> implements the following methods.
 
  $transaction->commit;
 
-Commits the transaction and returns the result.
+Commits an unmanaged transaction.
 
 After committing the transaction is closed and can no longer be used.
 
@@ -354,7 +362,7 @@ S<60 seconds.>
 
  $transaction->rollback;
 
-Rollbacks the transaction.
+Rolls back a transaction.
 
 After rolling back the transaction is closed and can no longer be
 used.
@@ -424,17 +432,19 @@ converted to strings before they are sent to the Neo4j server.
 
 This driver always reports all errors using C<die()>. Error messages
 received from the Neo4j server are passed on as-is.
+See L<Neo4j::Driver::Plugin/"error"> for accessing error details.
 
 Statement errors occur when the statement is executed on the server.
 This may not necessarily have happened by the time C<run()> returns.
-If you use C<try> to handle errors, make sure you use the
-L<Result|Neo4j::Driver::Result> within the C<try> block, for example
-by calling one of the methods C<fetch()>, C<list()> or C<summary()>.
+If you use L<C<try>|perlsyn/"Try"> to handle errors, make sure
+you actually I<use> the L<Result|Neo4j::Driver::Result> within
+the C<try> block, for example by retrieving a record or calling
+the method C<has_next()>.
 
 Transactions are rolled back and closed automatically if the Neo4j
 server encounters an error when running a query. However, if an
 I<internal> error occurs in the driver or in one of its supporting
-modules, explicit transactions remain open.
+modules, unmanaged transactions may remain open.
 
 Typically, no particular handling of error conditions is required.
 But if you wrap your transaction in a C<try> (or C<eval>) block,
